# Useful functions for C programmers

# Locate all things which look like function definitions in C, collect them up, and
# dump them into a new window as prototypes.

proc GetCPrototypes {window} \
{
	setmark $window temp;									# remember what was selected, we will not disturb it
	TextToBuffer tempFindBuffer {^[A-Za-z][A-Za-z0-9_* \t\n\[\]<>]*\([^\-{}=|!+/;]*\)[ \t]*;?};	# load up the expression (get existing prototypes as well as functions)

	setselectionends $window 0 0;							# move to the top of the buffer to perform the search
	if {[catch {findall $window tempFindBuffer -regex} message]==0} \
	{
		if {$message!=-1} \
		{
			set name [newbuffer [NewWindowName]];			# make a buffer to hold the prototypes
			copy $window $name;								# get selected prototypes into a buffer (leave each prototype individually selected)

			setmark $name temp;								# keep track of each prototype that was collected

			TextToBuffer tempFindBuffer {(^[ \t]*(static|typedef)[-]*)|([-]+\;$)};	# get rid of any static functions, or typedefs that look like functions, or function prototypes
			TextToBuffer tempReplaceBuffer {};
			catch {replaceall $name tempFindBuffer tempReplaceBuffer -regex -limitscope} message;	# do this, ignore any errors

			setselectionends $name 0 0;						# move to the top of the prototypes buffer

			TextToBuffer tempFindBuffer {(^[ \t]*\n)|([ \t]+$)};	# get rid of any empty lines, and any whitespace at the ends of lines
			TextToBuffer tempReplaceBuffer {};
			catch {replaceall $name tempFindBuffer tempReplaceBuffer -regex} message;	# do this, ignore any errors

			gotomark $name temp;							# select prototypes again

			TextToBuffer tempFindBuffer {^([-]+)};			# terminate all prototypes with a ;
			TextToBuffer tempReplaceBuffer {\0;};
			catch {replaceall $name tempFindBuffer tempReplaceBuffer -regex -limitscope} message;	# do this, ignore any errors

			# see if the last line is terminated with a new-line, if not, add one
			set end [lindex [textinfo $name] 1];				# get position of end of buffer
			if {[lindex [positiontolineoffset $name $end] 1]!=0} \
			{
				setselectionends $name $end $end;			# move to the end of the prototypes buffer
				insert $name "\n";							# terminate the last line
			}
			setselectionends $name 0 0;						# move to the top of the prototypes buffer

			flushundos $name;								# get rid of any undos we made here, just to be nice
			cleardirty $name;								# make it non-modified
			OpenDefaultWindow $name;						# give it to the user
			SetHighlightMode $name "C/C++";					# make it look like C code
			closemark $name temp;							# get rid of temp mark
		} \
		else \
		{
			beep
		}
	} \
	else \
	{
		okdialog $message
	}
	gotomark $window temp;									# put back the user's selection
	closemark $window temp;									# get rid of temp selection
}

# Locate all function like lines, bring them up in a list box, allowing the
# user to choose one, then move to the chosen one.
proc SelectCFunction {window} \
{
	setmark $window temp;									# remember what was selected, we will not disturb it
	TextToBuffer tempFindBuffer {^[A-Za-z][A-Za-z0-9_ *\[\]<>\t\n]*\([^\-{}=|!+/;]*\)};	# load up the expression
	setselectionends $window 0 0;							# move to the top of the buffer to perform the search
	if {[catch {findall $window tempFindBuffer -regex} message]==0} \
	{
		if {$message!=-1} \
		{
			set name [newbuffer [NewWindowName]];			# make a buffer to hold the prototypes
			copy $window $name;								# get selected prototypes into a buffer
			setselectionends $window 0 0;					# move to the top of the buffer
			setselectionends $name 0 0;						# move to the top of the prototypes buffer
			# see if the last line is terminated with a new-line, if not, add one
			set end [lindex [textinfo $name] 1];				# get position of end of buffer
			if {[lindex [positiontolineoffset $name $end] 1]!=0} \
			{
				setselectionends $name $end $end;			# move to the end of the prototypes buffer
				insert $name "\n";							# terminate the last line
			}

			set end [lindex [textinfo $name] 1];				# get position of end of buffer
			# see if the last line is terminated with a new-line, if not, add one
			if {[lindex [positiontolineoffset $name $end] 1]!=0} \
			{
				setselectionends $name $end $end;			# move to the end of the prototypes buffer
				insert $name "\n";							# terminate the last line
			}

			setselectionends $name 0 0;						# move to the top of the prototypes buffer
			flushundos $name;								# get rid of any undos we made here, just to be nice
			cleardirty $name;								# make it non-modified
			TextToBuffer tempFindBuffer {^[A-Za-z][^(;,\n]+\([^)]*\)};	# load up the expression
			if {[catch {findall $name tempFindBuffer -regex} message]==0} \
			{
				if {$message!=-1} \
				{
					catch {listdialog "Choose a Function:" [selectedtextlist $name]} choice
					if {[llength $choice]>=1} \
					{
						TextToBuffer tempFindBuffer [lindex $choice 0]
						setselectionends $window 0 0;				# move to the top of the buffer to perform the search
						catch {find $window tempFindBuffer} message
						HomeWindowToSelectionStart $window -strict
					} \
					else \
					{
						gotomark $window temp;				# put back the user's selection
					}
				} \
				else \
				{
					beep
					gotomark $window temp;					# put back the user's selection
				}
			} \
			else \
			{
				okdialog $message
				gotomark $window temp;						# put back the user's selection
			}
			closebuffer $name;								# get rid of the buffer
		} \
		else \
		{
			beep
			gotomark $window temp;							# put back the user's selection
		}
	} \
	else \
	{
		okdialog $message
		gotomark $window temp;								# put back the user's selection
	}
	closemark $window temp;									# get rid of temp selection
}

# Paste all of the selections in the clipboard as case statements in 'C'
# into window.
# This is handy when columnar selecting a bunch of enumerations, or
# defines to build a large switch statement from.
proc CasePaste {window} \
{
	breakundo $window
	foreach item [selectedtextlist [getclipboard]] \
	{
		insert $window "case $item:\n\tbreak;\n"
	}
	breakundo $window
}

addmenu {} LASTCHILD 1 "C" "" ""
	addmenu {C} LASTCHILD 1 "Get Prototypes"					{}				{GetCPrototypes [ActiveWindowOrBeep]}
	addmenu {C} LASTCHILD 1 "Locate Function..."				{}				{SelectCFunction [ActiveWindowOrBeep]}
	addmenu {C} LASTCHILD 0 "space1"							{\\S}			{}
	addmenu {C} LASTCHILD 1 "Case Paste"						{}				{CasePaste [ActiveWindowOrBeep]}
